package com.ruoyi.system.service.impl;

import com.ruoyi.system.mapper.WagesDetailsOriginalDataMapper;
import com.ruoyi.system.mapper.WagesOverviewOriginalDateMapper;
import com.ruoyi.system.model.wages.dao.WagesDetailsOriginalExcelDAO;
import com.ruoyi.system.model.wages.dao.WagesOverviewOriginalExcelDAO;
import com.ruoyi.system.model.wages.dto.WagesFormulaDto;
import com.ruoyi.system.model.wages.dto.WagesFormulaListDto;
import com.ruoyi.system.model.wages.vo.WagesFormulaVo;
import com.ruoyi.system.service.WagesCalculateConversionService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 2024/5/21
 */
@Service
@RequiredArgsConstructor
public class WagesCalculateConversionServiceImpl implements WagesCalculateConversionService {

    private final WagesDetailsOriginalDataMapper detailsOriginalDataMapper;
    private final WagesOverviewOriginalDateMapper originalDateMapper;
    private static final List<Character> FH = Arrays.asList('+', '-', '*', '/');

    @Override
    public List<WagesFormulaVo> conversionSurface(WagesFormulaListDto wagesFormulaListDto) {
        //根据中文映射相应的实体类字段
        Map<String, String> fieldMapping = new HashMap<>();
        fieldMapping.put("年", "year");
        fieldMapping.put("月", "month");
        fieldMapping.put("单位名称", "unitName");
        fieldMapping.put("人数", "peopleCount");
        fieldMapping.put("应付工资", "wagesPayable");
        fieldMapping.put("房租", "houseRent");
        fieldMapping.put("住宿", "accommodation");
        fieldMapping.put("暖气费", "heatingFee");
        fieldMapping.put("工会会费", "tradeUnionDues");
        fieldMapping.put("赔偿扣款", "compensationDeduction");
        fieldMapping.put("其他扣款", "otherDeductions");
        fieldMapping.put("互助医疗", "mutualMedicalAid");
        fieldMapping.put("通讯费", "communicationFee");
        fieldMapping.put("税金1", "tax1");
        fieldMapping.put("税金2", "tax2");
        fieldMapping.put("养老保险", "pensionInsurance");
        fieldMapping.put("失业保险", "unemploymentInsurance");
        fieldMapping.put("医疗保险", "medicalInsurance");
        fieldMapping.put("年金", "annuity");
        fieldMapping.put("住房公积金", "housingProvidentFund");
        fieldMapping.put("企业公积金", "enterpriseFund");
        fieldMapping.put("年金个税", "annuityIncomeTax");
        fieldMapping.put("代扣合计", "totalWithholding");
        fieldMapping.put("净支工资", "netSalary");
        fieldMapping.put("一孩费", "oneChildFee");
        fieldMapping.put("医疗补贴", "medicalSubsidy");
        fieldMapping.put("保健费", "healthCareFee");
        fieldMapping.put("班中餐", "classMealFee");
        fieldMapping.put("肥皂费", "soapFee");
        fieldMapping.put("护理费", "nursingFee");
        fieldMapping.put("防暑补贴", "heatstrokePreventionSubsidy");
        fieldMapping.put("取暖补贴", "heatingSubsidy");
        fieldMapping.put("稿费", "manuscriptFee");
        fieldMapping.put("司机出车补贴", "driverAllowance");
        fieldMapping.put("毕业生安家费", "graduateSettlementFee");
        fieldMapping.put("一次性伤残补助", "oneTimeDisabilitySubsidy");
        fieldMapping.put("讲课考务费", "lectureAndExamFee");
        fieldMapping.put("其他补贴", "otherSubsidies");
        fieldMapping.put("代付合计", "totalPayment");
        fieldMapping.put("工资总支款", "totalSalaryDisbursement");
        Map<String, String> fieldMapper = new HashMap<>();
        fieldMapper.put("应付工资", "totalWagesPayable");

        //定义一个集合用来，存储返回的字段
        List<WagesFormulaVo> voList = new ArrayList<>();
        // 获取前端传过来的集合（label, c, value）
        //详细表
        List<WagesDetailsOriginalExcelDAO> wagesDetails = detailsOriginalDataMapper.selectAll(wagesFormulaListDto.getDate());
        //略表
        List<WagesOverviewOriginalExcelDAO> wagesList = originalDateMapper.selectAll(wagesFormulaListDto.getDate());
        List<String> unitList = wagesList.parallelStream().map(WagesOverviewOriginalExcelDAO::getUnitName).collect(Collectors.toList());

        if (wagesList.size() == 0) {
            return null;
        }
        List<WagesFormulaDto> wagesFormulaDtoList = wagesFormulaListDto.getData();
        for (WagesFormulaDto wagesFormulaDto : wagesFormulaDtoList) {
            String value = wagesFormulaDto.getValue();
            String value1 = value;
            //解析公式value
            Map<String, List<String>> operandsMap = extractOperands(value);
            //只含公式中的字段
            List<String> formula = operandsMap.get("operands");
            try {
                //如果传过来的
                if (wagesFormulaDto.getValue().contains("人数")) {
                    // 第一轮处理wagesList
                    for (WagesOverviewOriginalExcelDAO dao1 : wagesList) {
                        WagesFormulaVo vo = new WagesFormulaVo();
                        vo.setLabel(wagesFormulaDto.getLabel());
                        vo.setDep(dao1.getUnitName());
                        vo.setValue(dao1.getTotalCount().toString());
                        vo.setWorker(dao1.getWorkerCount().toString());
                        vo.setCadre(dao1.getCadreCount().toString());
                        vo.setC(wagesFormulaDto.getC());
                        voList.add(vo);
                    }
                    List<WagesDetailsOriginalExcelDAO> collect = wagesDetails.parallelStream().filter(v -> !unitList.contains(v.getUnitName())).collect(Collectors.toList());
                    System.out.println(collect.size());
                    // 第二轮处理wagesDetails，过滤掉已处理过的部门
                    for (WagesDetailsOriginalExcelDAO dao : collect) {
                        WagesFormulaVo vo2 = new WagesFormulaVo();
                        vo2.setLabel(wagesFormulaDto.getLabel());
                        vo2.setDep(dao.getUnitName());
                        System.out.println(dao.getUnitName());
                        vo2.setValue(dao.getPeopleCount().toString());
                        vo2.setWorker("0");
                        vo2.setCadre("0");
                        vo2.setC(wagesFormulaDto.getC());
                        voList.add(vo2);
                    }
                }else if (wagesFormulaDto.getValue().contains("应付工资")) {
                    for (WagesOverviewOriginalExcelDAO dao : wagesList) {
                        Optional<WagesDetailsOriginalExcelDAO> first = wagesDetails.parallelStream().filter(v -> v.getUnitName().equals(dao.getUnitName())).findFirst();
                        WagesFormulaVo vo = new WagesFormulaVo();
                        if (first.isPresent()) {
                            vo.setLabel(wagesFormulaDto.getLabel());
                            vo.setDep(first.get().getUnitName());
//                            vo.setWorker(dao.getWorkerWagesPayable().toString());
                            vo.setCadre(dao.getCadreWagesPayable().toString());
                            vo.setC(wagesFormulaDto.getC());
                            for (String s : formula) {
                                if (fieldMapper.containsKey(s)) {
                                    String sqlResult = fieldMapper.get(s);
                                    Field wagesField = WagesOverviewOriginalExcelDAO.class.getDeclaredField(sqlResult);
                                    wagesField.setAccessible(true);
                                    Object fieldValue = wagesField.get(dao);
                                    value = StringUtils.replaceOnce(value, s, fieldValue.toString());
                                } else if (fieldMapping.containsKey(s)) {
                                    String sqlResult = fieldMapping.get(s);
                                    Field wagesField = WagesDetailsOriginalExcelDAO.class.getDeclaredField(sqlResult);
                                    wagesField.setAccessible(true);
                                    Object fieldValue = wagesField.get(first.get());
                                    value = StringUtils.replaceOnce(value, s, fieldValue.toString());
                                } else {
                                    // 如果传入的是数字而不是映射的字段，直接将数字作为 fieldValue 传递
                                    value = StringUtils.replaceOnce(value, s, s);
                                }
                            }
                           // vo.setValue(count(value));
                            vo.setValue(dao.getTotalWagesPayable().toString());
                            // 假设 count(value) 返回的是一个字符串，表示一个数值
                            String countString = count(value);
                            BigDecimal countValue = BigDecimal.valueOf(Double.parseDouble(countString));
                            BigDecimal cadreWagesPayable = dao.getCadreWagesPayable();
                            BigDecimal result = countValue.subtract(cadreWagesPayable);
                            vo.setWorker(result.toString());
                            voList.add(vo);
                            value = value1;
                        }
                    }
                    List<WagesDetailsOriginalExcelDAO> collect = wagesDetails.parallelStream().filter(v -> !unitList.contains(v.getUnitName())).collect(Collectors.toList());

                    for (WagesDetailsOriginalExcelDAO wagesDetail : collect) {
                        WagesFormulaVo vo = new WagesFormulaVo();
                        vo.setLabel(wagesFormulaDto.getLabel());
                        vo.setDep(wagesDetail.getUnitName());
                        vo.setWorker("0");
                        vo.setCadre("0");
                        vo.setC(wagesFormulaDto.getC());
                        for (String s : formula) {
                            String sqlResult = fieldMapping.get(s);
                            if (sqlResult == null) {
                                continue;
                            }
                            Field wagesField = WagesDetailsOriginalExcelDAO.class.getDeclaredField(sqlResult);
                            wagesField.setAccessible(true);
                            Object fieldValue = wagesField.get(wagesDetail);
                            value = StringUtils.replaceOnce(value, s, fieldValue.toString());
                        }
                        vo.setValue(count(value));

                        voList.add(vo);
                        value = value1;
                    }
                } else {
                    for (WagesDetailsOriginalExcelDAO wagesDetail : wagesDetails) {
                        WagesFormulaVo vo = new WagesFormulaVo();
                        vo.setLabel(wagesFormulaDto.getLabel());
                        vo.setDep(wagesDetail.getUnitName());
                        vo.setWorker("0");
                        vo.setCadre("0");
                        vo.setC(wagesFormulaDto.getC());
                        for (String s : formula) {
                            String sqlResult = fieldMapping.get(s);
                            Field wagesField = null;
                            wagesField = WagesDetailsOriginalExcelDAO.class.getDeclaredField(sqlResult);
                            wagesField.setAccessible(true);
                            Object fieldValue = wagesField.get(wagesDetail);
                            value = StringUtils.replaceOnce(value, s, fieldValue.toString());
                        }
                        vo.setValue(count(value));
                        voList.add(vo);
                        value = value1;
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        return voList;
    }

    private String count(String value) {
        value = value.replaceAll(" ", "");
        char sta = value.charAt(0);
        char end = value.charAt(value.length() - 1);
        if (sta < 48 || sta > 57) {
            return "公式填写有误";
        }
        if (end < 48 || end > 57) {
            return "公式填写有误";
        }
        return count1(value);
    }

    private String count1(String value) {
        Pattern pattern = Pattern.compile("^-?\\d+(\\.\\d+)?$");
        if (pattern.matcher(value).matches()) {
            return value;
        }
        // 判断是否为 x + y
        boolean ifAccordOne = true;
        // 判断是否为 x + y + z + ...
        boolean ifAccordTwo = true;
        // 记录第一个符合位置
        int accordPositionOne = -1;
        // 记录第二个符合位置
        int accordPositionTwo = value.length();
        for (int i = 0; i < value.length(); i++) {
            char charAt = value.charAt(i);
            if (FH.contains(charAt) && !ifAccordOne) {
                ifAccordTwo = false;
                accordPositionTwo = i;
                break;
            } else if (FH.contains(charAt)) {
                ifAccordOne = false;
                accordPositionOne = i;
            }
        }
        String sta = value.substring(0, accordPositionOne);
        String end = value.substring(accordPositionOne + 1, accordPositionTwo);
        char charAt = value.substring(accordPositionOne, accordPositionOne + 1).charAt(0);
        String jg = "";
        if (charAt == 42) {
            //*
            jg = new BigDecimal(sta).multiply(new BigDecimal(end)).toString();
        } else if (charAt == 43) {
            // +
            jg = new BigDecimal(sta).add(new BigDecimal(end)).toString();
        } else if (charAt == 45) {
            // -
            jg = new BigDecimal(sta).subtract(new BigDecimal(end)).toString();
        } else if (charAt == 47) {
            // /
            jg = new BigDecimal(sta).divide(new BigDecimal(end)).toString();
        }
        if (ifAccordTwo) {
            return jg;
        } else {
            String s = StringUtils.replaceOnce(value, value.substring(0, accordPositionTwo), jg);
            value = count1(s);
        }
        return value;
    }


    //解析公式
    private Map<String, List<String>> extractOperands(String value) {
        Map<String, List<String>> operandsMap = new HashMap<>();
        List<String> operands = new ArrayList<>();
        List<String> operators = new ArrayList<>();

        StringBuilder operandBuilder = new StringBuilder();
        for (char c : value.toCharArray()) {
            if (isOperator(c)) {
                if (operandBuilder.length() > 0) {
                    operands.add(operandBuilder.toString().trim());
                    operandBuilder = new StringBuilder();
                }

                operators.add(String.valueOf(c));
            } else {
                operandBuilder.append(c);
            }
        }

        if (operandBuilder.length() > 0) {
            operands.add(operandBuilder.toString().trim());
        }

        operandsMap.put("operands", operands);
        operandsMap.put("operators", operators);

        return operandsMap;
    }

    //判断是否是操作符
    private boolean isOperator(char c) {
        return c == '+' || c == '-' || c == '*' || c == '/';
    }
}